// ==================================================================
//
// CBP_3352_LinearCamera_2.c
//
// 13-02-21
// Now returns color seen and writes trajectory and omegas to Octave
// file for automatic plotting.
//
// ==================================================================
 
// Included libraries
#include <webots/camera.h>
#include <webots/display.h>
#include <webots/robot.h>
#include <webots/gps.h>

#include <stdio.h>
#include <stdlib.h>

#include "CBP_3352_Helper1.h"
#include "CBP_3352_Helper2.h"
#include "CBP_3352_Helper3.h"

// Global defines
#define TIME_STEP 16
#define WHEEL_RADIUS 0.0205
#define AXLE_LENGTH 0.053
#define MAX_SPEED 6.28

// Variables related to the camera and extra display
WbDeviceTag cam;
static WbDeviceTag display1;
unsigned short width, height;
 
// Tag for the gps
WbDeviceTag gps;

double omegaL, omegaR;

// Function to set up the camera
void init_camera(){
  cam = wb_robot_get_device("camera");
  wb_camera_enable(cam, TIME_STEP);
  width = wb_camera_get_width(cam);
  height = wb_camera_get_height(cam);
  printf("Camera width = %d, height = %d\n",width,height);
  // get the display device
  display1 = wb_robot_get_device("display");
}

// Function to set up the gps
void init_gps(){
  gps = wb_robot_get_device("gps");
  wb_gps_enable(gps, TIME_STEP); 
}

// -------------------------------------------------------------------
 
int main() {

  int nr_pels_detected;
  double colorFound;
  double time = 0;
  
  FILE* octaveFile;
  octaveFile = fopen("Robot.m", "w");
  writeOctaveHead(octaveFile); 
  
  wb_robot_init();
  
  cbp_motors_init();
  init_camera();
  init_gps();

  unsigned char *lineImage = (unsigned char *)malloc(sizeof(unsigned char) * width * height * 8);

  while (wb_robot_step(TIME_STEP) != -1) {
   
    const unsigned char *image;
 
    image = wb_camera_get_image(cam);

    nr_pels_detected = imageSegment(image,lineImage, width, height,&colorFound);
    printf("nr_pels_detected = %d, Color is %f\n",nr_pels_detected,colorFound);
    WbImageRef iref = wb_display_image_new(display1, width, height, lineImage, WB_IMAGE_BGRA);
    wb_display_image_paste(display1, iref, 0, 0, false);
    
    // 2. Generate control signal from image
    double line_center = (double)getAvLocation(lineImage,width,height);
    
    double gain = 7.0;
    double error = (width/2 - line_center)/width;
    
    if(nr_pels_detected < 10) error = 0;
    
    double fwdFactor = 1.5;
    omegaL = (fwdFactor - gain*error)*0.5*MAX_SPEED;
    omegaR = (fwdFactor + gain*error)*0.5*MAX_SPEED;

    // clamp the omegas if they become too large
    if(omegaL >  MAX_SPEED) omegaL =  MAX_SPEED;
    if(omegaL < -MAX_SPEED) omegaL = -MAX_SPEED;
    if(omegaR >  MAX_SPEED) omegaR =  MAX_SPEED;
    if(omegaR < -MAX_SPEED) omegaR = -MAX_SPEED;
        
    printf("line_center = %f, percent error = %f, omegaL = %f, omegaR = %f\n",line_center,error*100,omegaL,omegaR);    

    // Write stuff to the Octave file (x,y) from gps
    double x,y;
    const double *gpsVals2 = wb_gps_get_values(gps); 
    x = gpsVals2[2];
    y = gpsVals2[0];       
    time += (double)TIME_STEP/(double)1000;    
    writeDataToOctave(octaveFile,time,x,y,omegaL,omegaR);
    
    cbp_motors_set_velocity(omegaL,omegaR);
        
  }
  free(lineImage);
  
  writeOctaveTail(octaveFile);
  fclose(octaveFile);
  wb_robot_cleanup();

  return 0;
}

// -------------------------------------------------------------------
