// ===========================================================================
//
// CBP_Quadruped_1.c
//
// CBP 15-11-20
//
// ===========================================================================

#include <math.h>
#include <stdio.h>
#include <webots/keyboard.h>
#include <webots/motor.h>
#include <webots/robot.h>

// ODE Solvers
#include "rkf45.h"
#include "CBP_VderPol.h"
#include "CBP_Helper_1.h"

// control step duration in milliseconds
#define TIME_STEP 16

#define NEQN 8

int main() {
 
  WbDeviceTag motors[6];
  const char *MOTOR_NAMES[] = {"hip0", "hip1", "hip2", "hip3", "spine", NULL};
          
  // rkf stuff
  double abserr;
  int flag;
  double relerr;
  double t;
  double t_out;
  double y[NEQN];
  double dydt[NEQN];

  double test[] = {0.5, 0.5};
  double rot[2];
  double test2[2];
  
  double theta1, theta2, theta3;  
  
  FILE* logFile;
  
  wb_robot_init();
    
  logFile = fopen("Robot.m","w");
  writeOctaveHead(logFile);
  
  // DEBUG: Testing the rotation and addition routines
  rotate(M_PI/2,test,rot);
  printf("Rot(x,y) = %f %f\n",rot[0],rot[1]);
  add(test,rot,test2);
  printf("Test2 (x,y) = %f %f\n",test2[0],test2[1]);
  
  // Here the walk parameters
  theta1 = 0.25*2.0*M_PI;
  theta2 = 0.25*2.0*M_PI;
  theta3 = 0.5*2.0*M_PI;   
  
    // Here the trot parameters
  theta1 = 0.5*2.0*M_PI;
  theta2 = 0.0;
  theta3 = 0.5*2.0*M_PI;  
  
  /*
  // Here the bound parameters
  theta1 = 0.5*2.0*M_PI;
  theta2 = 0.5*2.0*M_PI;
  theta3 = 0.0; 
  */
     
  // get motor device tags
  int i;
  for (i = 0; MOTOR_NAMES[i]; i++) {
    motors[i] = wb_robot_get_device(MOTOR_NAMES[i]);
  }

  // Calculate the epsilon
  abserr = sqrt ( r8_epsilon ( ) );
  relerr = sqrt ( r8_epsilon ( ) );
  printf("Root epsilon: %e\n",abserr);
  
  flag = 1;

  // Set some initial conditions
  y[0] = 0.1;
  y[1] = 1.0;
  y[2] = 0.3;
  y[3] = 0.11;
  y[4] = 0.1;
  y[5] = 0.6;
  y[6] = 0.11;
  y[7] = 2.1;
       
  setParams3(10,1.0,1.7*2.0*M_PI,theta1,theta2,theta3);
   
  // Initialize the ODE Solver                 
  t = 0.0;
  t_out = TIME_STEP/1000.0; 
  vdPol3 ( t, y,  dydt );
    
  while (wb_robot_step(TIME_STEP) != -1) {
                      
    flag = r8_rkf45 ( vdPol3, NEQN, y, dydt, &t, t_out, &relerr, abserr, flag );
      
      
  //  printf ( "%4d  %12f  %12f  %12f  %12f  %12f\n", flag, t, y[0], y[2], y[4], y[6] );
 
    t_out = t + TIME_STEP/1000.0;
    
    double phaseLF = y[0];
    double phaseLH = y[2];
    double phaseRH = y[4];
    double phaseRF = y[6];
      
      // Chosen empirically     
    double scalingF = 0.25;
    double scalingH = 0.15;
    double offsetH = -0.2;
    double offsetF = 0.0;
            
    phaseLF = phaseLF*scalingF + offsetF;
    phaseLH = phaseLH*scalingH + offsetH;      
    phaseRH = phaseRH*scalingH + offsetH;      
    phaseRF = phaseRF*scalingF + offsetF;      

    writeData2ToOctave(logFile,t,phaseLF,phaseLH);

    if(t > 2.0) {
      wb_motor_set_position(motors[2], phaseLF);
      wb_motor_set_position(motors[3], phaseLH);      
      wb_motor_set_position(motors[1], phaseRH);    
      wb_motor_set_position(motors[0], phaseRF);
    }

  }
  
  writeOctaveTail(logFile);

  wb_robot_cleanup();

  return 0;
}
